local addonName, addonData = ...

local prefix, slashes, quests = "wh", {"cb", "candy", "candybucket", "candybuckets"}, {}
local continentDictionary = {}

local addon = CreateFrame("Frame")
addon:RegisterEvent("ADDON_LOADED")
addon:SetScript("OnEvent", function(addon, event, name)
  if event == "ADDON_LOADED" and addonName == name then
    addon:UnregisterEvent(event)
    addon:Init()
  elseif event == "QUEST_QUERY_COMPLETE" then
    if addon.report then
      addon.report = nil
      addon:Report()
    end
  end
end)

function addon:Init()
  addon:RegisterEvent("QUEST_QUERY_COMPLETE")
  local _, faction = UnitFactionGroup("player")

  local continents = {GetMapContinents()}
  for contId, contName in pairs(continents) do
    local zones = {GetMapZones(contId)}
    for _, zoneName in pairs(zones) do
      continentDictionary[zoneName] = contName
    end
    zones = nil
  end
  continents = nil

  do
    for questId, areaId in pairs(addonData[faction]) do
      table.insert(quests, {a=areaId, n=GetMapNameByID(areaId), q=questId})
    end
    for questId, areaId in pairs(addonData.Both) do
      table.insert(quests, {a=areaId, n=GetMapNameByID(areaId), q=questId})
    end
    table.sort(quests, function(a, b)
      local contA = continentDictionary[a.n]
      local contB = continentDictionary[b.n]
      return contA < contB or (contA == contB and a.n < b.n) or false
    end)
  end
  for i, suffix in pairs(slashes) do
    _G["SLASH_"..addonName..i] = "/"..prefix..suffix
  end
  SlashCmdList[addonName] = function()
    if not addon.report then
      addon.report = 1
      QueryQuestsCompleted()
    else
      DEFAULT_CHAT_FRAME:AddMessage("There is a 15 minutes cooldown each time you use this command, be patient! This is because Blizzard limits how often you can ask the server what quests you have completed.", 1, .8, 0)
    end
  end
end

function addon:Report()
  local done, c, area, name, quest = GetQuestsCompleted(), 1
  if #quests > 0 then
    DEFAULT_CHAT_FRAME:AddMessage("You are missing the following Candy Buckets:", 1, 1, 0)
    local lastCont = ""
    for i = 1, #quests do
      area, name, quest = quests[i].a, quests[i].n, quests[i].q
      if not done[quest] then
        local thisCont = continentDictionary[name]
        if thisCont ~= lastCont then
          DEFAULT_CHAT_FRAME:AddMessage(" - "..thisCont.." - ", 1, .8, 0)
        end
		lastCont = thisCont
        DEFAULT_CHAT_FRAME:AddMessage("["..c.."] |cffffff00|Hquest:"..quest..":-1|h[Candy Bucket]|h|r in "..name, 1, .6, 0)
        c = c + 1
      end
    end
    DEFAULT_CHAT_FRAME:AddMessage("There are "..(c-1).." more buckets to go!", 1, .8, 0)
  else
    DEFAULT_CHAT_FRAME:AddMessage("You have obtianed all the candy possible from the Candy Buckets, amazing!", 0, 1, 0)
  end
end
